@@ -10,7 +10,7 @@
 /* global ReloadMessage: false, gDBView: false, gSignatureStatus: false, gEncryptionStatus: false, showMessageReadSecurityInfo: false */
 /* global gFolderDisplay: false, messenger: false, currentAttachments: false, msgWindow: false, PanelUI: false */
 /* global currentHeaderData: false, gViewAllHeaders: false, gExpandedHeaderList: false, goDoCommand: false, HandleSelectedAttachments: false */
-/* global statusFeedback: false, displayAttachmentsForExpandedView: false, gMessageListeners: false, gExpandedHeaderView */
+/* global statusFeedback: false, displayAttachmentsForExpandedView: false, gMessageListeners: false, gExpandedHeaderView: false, gSignedUINode: false */
 
 var EnigmailCompat = ChromeUtils.import("chrome://enigmail/content/modules/compat.jsm").EnigmailCompat;
 var EnigmailCore = ChromeUtils.import("chrome://enigmail/content/modules/core.jsm").EnigmailCore;
@@ -112,15 +112,9 @@ Enigmail.msg = {
     }
 
     let t = document.getElementById("tabmail");
-
     if (t) {
       // TB >= 63
-      t.addEventListener("pageshow", function(e) {
-        if (e.type === "pageshow" && e.target.URL === "about:preferences") {
-          let Overlays = ChromeUtils.import("chrome://enigmail/content/modules/overlays.jsm", {}).Overlays;
-          Overlays.loadOverlays("Enigmail", e.target.defaultView, ["chrome://enigmail/content/ui/enigmailPrivacyOverlay.xul"]);
-        }
-      }, false);
+      t.addEventListener("pageshow", Enigmail.msg.pageShowListener, false);
     }
 
     let customizeToolbar = document.getElementById("customizeToolbarSheetIFrame");
@@ -227,6 +221,14 @@ Enigmail.msg = {
     Enigmail.msg.messageListener.onEndHeaders();
   },
 
+  pageShowListener: function(e) {
+    if (e.type === "pageshow" && e.target.URL === "about:preferences") {
+      EnigmailLog.DEBUG("enigmailMessengerOverlay.js: loading enigmailPrivacyOverlay.xul\n");
+      let Overlays = ChromeUtils.import("chrome://enigmail/content/modules/overlays.jsm", {}).Overlays;
+      Overlays.loadOverlays("Enigmail", e.target.defaultView, ["chrome://enigmail/content/ui/enigmailPrivacyOverlay.xul"]);
+    }
+  },
+
   messageListener: {
     onStartHeaders: function() {
       Enigmail.msg.mimeParts = null;
@@ -731,6 +733,8 @@ Enigmail.msg = {
         };
       }
 
+      this.hideNonStandardMimeStructure(mimeMsg);
+
       // Copy selected headers
       Enigmail.msg.savedHeaders = {
         autocrypt: []
@@ -919,6 +923,27 @@ Enigmail.msg = {
       return -1;
   },
 
+  hideNonStandardMimeStructure: function(mimeMsg) {
+    if (Enigmail.msg.securityInfo === null) return;
+    if (Enigmail.msg.securityInfo.encryptedMimePart === "1") return;
+    if (!((Enigmail.msg.securityInfo.statusFlags & EnigmailConstants.PGP_MIME_SIGNED) !== 0 &&
+        (Enigmail.msg.securityInfo.statusFlags & EnigmailConstants.DECRYPTION_OKAY) === 0)) return;
+
+    if (Enigmail.msg.securityInfo.encryptedMimePart === "1.1" &&
+      mimeMsg.subParts.length === 2 &&
+      mimeMsg.headers.contentType.type === "multipart/mixed" &&
+      mimeMsg.subParts[1].headers.contentType.type === "text/plain") return;
+
+    // Hide status bar for PGP/MIME signed messages, if we found a non-standard message structure
+    Enigmail.msg.securityInfo.statusFlags = 0;
+    Enigmail.msg.securityInfo.statusLine = "";
+    Enigmail.msg.securityInfo.statusInfo = "";
+    Enigmail.hdrView.displayStatusBar();
+    let signedUINode = Enigmail.hdrView.getSignedIcon();
+    signedUINode.removeAttribute("signed");
+    signedUINode.collapsed = true;
+  },
+
   // display header about reparing buggy MS-Exchange messages
   buggyMailHeader: function() {
     let uriStr = EnigmailURIs.createMessageURI(this.getCurrentMsgUrl(),
@@ -2738,6 +2763,10 @@ Enigmail.msg = {
     window.removeEventListener("unload", Enigmail.msg.messengerClose, false);
     window.removeEventListener("unload-enigmail", Enigmail.msg.onUnloadEnigmail, false);
     window.removeEventListener("load-enigmail", Enigmail.msg.messengerStartup, false);
+    let t = document.getElementById("tabmail");
+    if (t) {
+      t.removeEventListener("pageshow", Enigmail.msg.pageShowListener, false);
+    }
 
     this.messageCleanup();
 
